//******************************************************************************
//  poolthermometer.c - Measure temperature and display by flashing LEDs
//  Based on a sample program by
//  L. Westlund
//  Texas Instruments Inc.
//  May 2006
//  Built with CCE Version: 3.2.0 and IAR Embedded Workbench Version: 3.41A
//******************************************************************************
#include  "msp430x20x2.h"

const unsigned int delay = 18;  // Corresponds to about one ms
// max value of TACCR0 = 65535 so max delay is 65535/18=3640 ms = 3.640 sec

#define NLIGHTS 6
unsigned char onoff = 0;
unsigned char whichLight = 0;  // count from 0 to NLIGHTS, index into P1light
const unsigned char P1light[NLIGHTS] = {0x01, 0x80, 0x10, 0x04, 0x40, 0x08};

// Set the current LED (as determined by whichLight) on or off (as determined
// by onoff) and wait the number of ms in the parameter. Precise timing is
// not critical in this application.
void setLEDandWait(unsigned int ms)
{
   for (int i=0; i<10; i++)
   {
      TACCR0 = delay*ms;
      TACCTL0 |= CCIE;                          // Compare-mode interrupt.
      // TACLK = SMCLK/8;; SMCLK = DCOCLK
      // set timer for 8*30 TACLK = 8*8*30 TACLK
      // DCOCLK ~1.1 MHz, 1 clock ~ 0.9 us
      // set timer for 8*8*18*0.9 =  1036.8 us = 1.0368 ms
      TACTL = TASSEL_2 | MC_1 | ID_3;           // TACLK = SMCLK, Up mode, divide by 8.
      if (onoff)
      {
         // Turn on one light determined by whichLight
         P1OUT = P1light[whichLight];
      }
      else
      {
         // Turn off all lights
         P1OUT = 0;
      }
      LPM0;                                     // Wait for timer.
   }
   TACCTL0 &= ~CCIE;                         // Disable timer Interrupt
}

// Read the temperature using the internal thermometer using the ADC10 and
// return the raw value 0 to 1023.
unsigned readADC10temp(void)
{
   // Temp Sensor ADC10CLK/4
   ADC10CTL1 = INCH_10 + ADC10DIV_3;
   // SREF = Vr+=Vref, Vr- = Vss
   // sample hold time 64 clocks
   // reference output on
   // ADC10 on
   // Enable interrupt
   ADC10CTL0 = SREF_1 + ADC10SHT_3 + REFON + ADC10ON + ADC10IE;
   setLEDandWait(1);                              // Delay to allow Ref to settle

   ADC10CTL0 |= ENC + ADC10SC;             // Sampling and conversion start
   LPM0;                                  // wait for conversion complete interrupt

   unsigned temp = ADC10MEM;
   // Done with ADC10, turn off ENC.
   ADC10CTL0 = 0;
   // Turn off everything else
   ADC10CTL0 = 0;
   return temp;
}

unsigned long IntDegFx342;
unsigned long IntDegFx20;
unsigned long IntDegFx10;
unsigned long OldIntDegFx20 = 0;
unsigned char hund, tens, ones, tenths;
int temp100;

// Read the temperature 342 times, average the readings and convert to
// degrees F times 20. If close to previous temperature, retain old
// value. If different enough, return temperature in global variable.
void readTemperature(void)
{
   // oC = ((A10/1024)*1500mV)-986mV)*1/3.55mV = A10*423/1024 - 278
   //IntDegC = ((temp - 673) * 423) / 1024;
   //--------------------------------------------------------------
   // oF = ((A10/1024)*1500mV)-923mV)*1/1.97mV = A10*761.42/1024 - 468.5
   //IntDegF = (temp - 630) * 761.42 / 1024;
   // IntDegFx10 = (temp-630) * 7614.2 / 1024
   // 7614.2/1024 = 171/23
   IntDegFx342 = 0;
   for (int i=0; i<2*171; i++)
   {
      IntDegFx342 += readADC10temp()-630;
   }
   IntDegFx20 = IntDegFx342 / 23;
   if (IntDegFx20 < OldIntDegFx20-2 || IntDegFx20 > OldIntDegFx20+2 )
   {
      OldIntDegFx20 = IntDegFx20;
      IntDegFx10 = IntDegFx20/2;
   }
}

// Display a single digit by turning led on and off with
// appropriate durations

void display (unsigned char count)
{
   if (count == 0)
   {
      onoff = 1;
      setLEDandWait(550);
      onoff = 0;
      setLEDandWait(400);
   }
   else
   {
      while (count > 0)
      {
         onoff = 1;
         setLEDandWait(100);
         onoff = 0;
         setLEDandWait(400);
         count--;
      }
   }
   setLEDandWait(800);
}

// Initialize, read temperature, convert to decimal, and display
// by flashing light.
void main(void)
{
   WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
   P1DIR = 0xFF;  // output direction
   P2DIR = 0xFF;  // output direction
   P1OUT = 0;     // all lights off
   P2OUT = 0;     // all lights off
   __enable_interrupt();
   while(1)
   {
      readTemperature();
      temp100 = IntDegFx10;
      hund = temp100/1000;
      if (hund > 0)
         display(hund);
      temp100 -= hund * 1000;
      tens = temp100/100;
      display(tens);
      temp100 -= tens*100;
      ones = temp100/10;
      display(ones);
      temp100 -= ones*10;
      tenths = temp100;
      if (tenths)
      {
         setLEDandWait (135);
         onoff = 1;
         setLEDandWait (15);     // decimal point
         onoff = 0;
         setLEDandWait (800);
         display (tenths);
      }
      setLEDandWait(2000);
      whichLight++;   // cycle through lights
      if (whichLight == NLIGHTS)
         whichLight = 0;
   }
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
  __bic_SR_register_on_exit(CPUOFF);        // Clear CPUOFF bit from 0(SR)
}

#pragma vector=TIMERA0_VECTOR
__interrupt void ta0_isr(void)
{
  TACTL = 0;
  LPM0_EXIT;                                // Exit LPM0 on return
}
